/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup debug
 * @{
 */
/** @file
 */

#include <stdio.h>
#include <stdlib.h>

#include "cons.h"

static char *line_buffer;
static int line_chars;

static int line_active = 0;

static void line_show(void)
{
	int i;

	printf("> ");
	for (i = 0; i < line_chars; ++i)
		putchar(line_buffer[i]);

	fflush(stdout);
}

static void line_hide(void)
{
	int i;

	printf("\b\b");
	for (i = 0; i < line_chars; ++i)
		putchar('\b');
}

void cons_printf(const char *fmt, ...)
{
	va_list ap;

	if (line_active) line_hide();
	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);
	if (line_active) line_show();
}

void cons_read_line(char *buffer, int n)
{
	char c;

	line_buffer = buffer;
	line_chars = 0;

	line_active = 1;
	line_show();

	line_chars = 0;
	while (line_chars < n - 1) {
		c = getchar();
		if (c == '\n') break;
		if (c == '\b') {
			if (line_chars > 0) {
				putchar('\b');
				--line_chars;
			}
			continue;
		}
		
		putchar(c);
		buffer[line_chars++] = c;
	}

	putchar('\n');
	buffer[line_chars] = '\0';

	line_active = 0;
}

/** @}
 */
